package terrain;

import gl4java.GLContext;
import gl4java.awt.GLAnimCanvas;
import gl4java.awt.GLCanvas;
import gl4java.utils.glut.*;

/***********************************************
 * TerrainCanvas
 * date: 2004-05-18
 * author: Kristian Lunde
 * This class has all the responsiblility for drawing openGL objects/heightfields to the canvas/screen.
 **********************************************/

public class TerrainCanvas extends GLAnimCanvas {
	
	private GLUTFunc glut = null;
		
	private float pos_x = 25.0f;
	private float pos_y = 13.0f;
	private float pos_z = 13.0f;
	
	private float rot_x = 0.0f;
	private float rot_z = 0.0f;
	private float rot_y = 0.0f;
	private float last_x = 0.0f;
	private float last_z = 0.0f;
	private float delta_x = 1.0f;
	private float delta_z = 1.0f;
	
	private HeightField heightField;
	
	/**
	 * Constructor
	 * @param int w - is the width of the canvas.
	 * @param int h - is the height of the canvas.
	 */
	public TerrainCanvas(int w, int h){ super(w, h);  }
	
	/* preInit */
	public void preInit(){
		doubleBuffer = true;
  	stereoView = false;
  	accumSize = 8;
	}
		
	/**
	 * init
	 * Setup lightsource, backgroundcolor.
	 */
	public void init(){
		glut = new GLUTFuncLightImpl(gl, glu);
		this.reshape(this.getSize().width, this.getSize().height);
			
		//setup lightsource
		float ambient[] = {0.0f, 0.0f, 1.0f, 1.0f};
  	float diffuse[] = {0.0f, 1.0f, 0.0f, 1.0f};
  	float position[] ={0.0f, 1.0f, 3.0f, 1.0f};
  	//float lmodel_ambient[] = {0.4f, 0.4f, 0.4f, 1.0f};
  	float lmodel_ambient[] = {1.0f, 1.0f, 1.0f, 1.0f};
  		
  	gl.glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
  	gl.glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
  	gl.glLightfv(GL_LIGHT0, GL_POSITION, position);
  		
  	gl.glEnable(GL_LIGHT0);
  	gl.glEnable(GL_LIGHTING);
			
		//smooth the drawing
  	gl.glShadeModel(GL_SMOOTH);
  		
  	//normailze normals
  	gl.glEnable(GL_NORMALIZE);
  		
  	//depth sorting
  	gl.glEnable(GL_DEPTH_TEST);
  	gl.glDepthFunc(GL_LESS);
			
		//backgroundcolor	- black
		gl.glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
	}
		
	/**
	 * display
	 * load the displaymatrix, setup the "eye" and call the drawHeightField method.
	 */
	public void display(){
		if(glj.gljMakeCurrent() == false)
			return;
		
		gl.glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  	gl.glClear(GL_ACCUM_BUFFER_BIT);	
  	gl.glLoadIdentity();

		glu.gluLookAt(pos_x, pos_y, pos_z,
  							 0.0f, 0.0f, 0.0f,
  								0.0f, 1.0f, 0.0f);

		drawHeightField();

		gl.glFlush();
			
		glj.gljSwap();
		glj.gljFree();
	}
	
	/**
	 * drawHeightField
	 * setup the heightfield materials, and colors. Set the rotation of the heightfield, and call the draw method.
	 */		
	private void drawHeightField(){	
		
		float amb[] = {1.0f, 1.0f, 1.0f, 1.0f};
  	float diff[] = {0.0f, 0.5f, 0.0f, 1.0f};
  	float spec[] = {0.0f, 0.0f, 0.0f, 1.0f};
  	float shine = 0.25f;
  	gl.glMaterialfv(GL_FRONT, GL_AMBIENT, amb);
  	gl.glMaterialfv(GL_FRONT, GL_DIFFUSE, diff);
  	gl.glMaterialfv(GL_FRONT, GL_SPECULAR, spec);
  	gl.glMaterialf(GL_FRONT, GL_SHININESS, shine*128.0f);
	
		gl.glRotatef(rot_x, 1.0f, 0.0f, 0.0f);
		gl.glRotatef(rot_z, 0.0f, 1.0f, 0.0f);
		
		draw(heightField.getSize() - 1, heightField.getSize() - 1);
	}
	
	/**
	 * draw
	 * @param int x - the matrix x size;
	 * @param int y - the matrix y size;
	 * Recursive method to draw the heightfield, calls the drawTriangle and drawOppositeTriangle methods.
	 */
	private void draw(int x, int y){
		if(x == 0) return;

		gl.glBegin(GL_LINE_STRIP);
			drawOppositeTriangle(x, y);
		gl.glEnd();
		gl.glBegin(GL_LINE_STRIP);
			drawTriangle(x, y);
		gl.glEnd();
		draw(x - 1, y);
	}
	
	/**
	 * drawTriangle
	 * @param int x - is the size of the matrix.
	 * @param int y - is the size of the matrix.
	 * add vertexes from the heightfield to the canvas.
	 */
	private void drawTriangle(int x, int y){
		if(y == 0) return;
		
		gl.glVertex3f(heightField.getWidth(x, y), heightField.getHeight(x, y), heightField.getLength(x, y));
		gl.glVertex3f(heightField.getWidth(x-1, y), heightField.getHeight(x-1, y), heightField.getLength(x-1, y));
		gl.glVertex3f(heightField.getWidth(x-1, y-1), heightField.getHeight(x-1, y-1), heightField.getLength(x-1, y-1));
		
		drawTriangle(x, y - 1);
	}
	
	/**
	 * drawOppositeTriangle
	 * @param int x - is the size of the matrix.
	 * @param int y - is the size of the matrix.
	 * add vertexes from the heightfield to the canvas.
	 */
	private void drawOppositeTriangle(int x, int y){
		if(y == 0) return;

		gl.glVertex3f(heightField.getWidth(x, y), heightField.getHeight(x, y), heightField.getLength(x, y));
		gl.glVertex3f(heightField.getWidth(x, y-1), heightField.getHeight(x, y-1), heightField.getLength(x, y-1));
		gl.glVertex3f(heightField.getWidth(x-1, y-1), heightField.getHeight(x-1, y-1), heightField.getLength(x-1, y-1));
	
		drawOppositeTriangle(x, y -1);
	}
	
	/**
	 * reshape
	 * @param int w - is the new width of the canvas.
	 * @param int h - is the new height of the canvas.
	 */
	public void reshape(int w, int h){
  	gl.glViewport(0, 0, w, h);
  	gl.glMatrixMode(GL_PROJECTION);
  	gl.glLoadIdentity();
  	glu.gluPerspective(45.0, (float) w/h, 1.0, 500.0);
  	gl.glMatrixMode(GL_MODELVIEW);	
  }
  
  /**
   * setHeightField
   * @param HeightField hf - set the heightfield to be drawed on the canvas.
   */	
  public void setHeightField(HeightField hf){ heightField = hf; }	
  
  /**
	 * setRotX
	 * @param float x - set rot_x = x;
	 */
  public void setRotX(float x){ rot_x = x; }
  
  /**
	 * getRotX
	 * @return float the rot_x value;
	 */
  public float getRotX(){ return rot_x; }
	
	 /**
	 * setRotZ
	 * @param float z - set rot_z = z;
	 */
	public void setRotZ(float z){ rot_z = z; }
	
	/**
	 * getRotY
	 * @return float the rot_y value;
	 */
	public float getRotZ(){ return rot_z; }
	
	 /**
	 * setRotY
	 * @param float y - set rot_y = y;
	 */
	public void setRotY(float y){ rot_y = y; }
	
	/**
	 * getRotY
	 * @return float the rot_y value;
	 */
	public float getRotY(){ return rot_y; }
	
	/**
	 * setLastX
	 * @param float x - set the last_x value like x.
	 */
	public void setLastX(float x){ last_x = x; }

	/**
	 * setLastZ
	 * @param float z - set the last_z value like z.
	 */	
	public void setLastZ(float z){ last_z = z; }
	
		/**
	 * setDeltaX
	 * @param float x - set the delta_x value like x.
	 */
	public void setDeltaX(float x){ delta_x = x; }
	
		/**
	 * setLastZ
	 * @param float z - set the delta_z value like z.
	 */
	public void setDeltaZ(float z){ delta_z = z; }
	
	/**
	 * getLastX
	 * @return float the last_x value;
	 */
	public float getLastX(){ return last_x; }
	
	/**
	 * getLastZ
	 * @return float the last_z value;
	 */
	public float getLastZ(){ return last_z; }
	
	/**
	 * getDeltaX
	 * @return float the delta_x value;
	 */
	public float getDeltaX(){ return delta_x; }
	
	/**
	 * getDeltaZ
	 * @return float the delta_z value;
	 */
	public float getDeltaZ(){ return delta_z; }
	
	/**
	 * setPosX
	 * @param float x - set the pos_x value = x.
	 */
	public void setPosX(float x){ pos_x = x; }
	
	/**
	 * getPosX
	 * @return float the pos_x value;
	 */
	public float getPosX(){ return pos_x; }
}
		

		
			

		
