package terrain;

import javax.swing.UIManager;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import terrain.filter.*;

/*******************************************************************
 * TerrainViewer
 * date: 2004-05-16
 * author: Kristian Lunde
 * This is the main class for the terrain/smoothing system. It administrate all objects.
 ******************************************************************/

public class TerrainViewer extends JFrame implements ActionListener{

	private HeightField heightField;
	private TerrainCanvas terrainCanvas;
	private TerrainLoader terrainLoader;
	
	private Filter filter;
	private BandFilter bandFilter;
	private LowpassFilter lowpassFilter;
	private HighpassFilter highpassFilter;
	private MeanFilter meanFilter;
	private GaussianFilter gaussianFilter;
	private KuwaharaFilter kuwaharaFilter;
	
	private WindowManager winManager;
	private MouseManager mouseManager;
	private KeyManager keyManager;
	private MenuBar viewMenuBar;
	private JPanel statusBar, input_panel;
	private JLabel message_label, input_label, iteration_label;
	private JTextField input_field, iteration_field;
	private JButton smooth_button;
	
	/* Load in a default terrain */
	private String file_path = "terrain\\terrain_1.ter";
	
	private MenuItem open_item, exit_item;
	private MenuItem bandfilter_item, lowpassfilter_item, highfilter_item, meanfilter_item, 
										gaussianfilter_item, kuwaharafilter_item, remove_filtering_item;
	
	/**
	 * Kickstart the application
	 */
	public static void main(String[] args){ new TerrainViewer(); }

	/**
	 * Constructor
	 * Create all objects needed
	 */
	public TerrainViewer(){
		//Set windows look and feel on the GUI.
		try {
        UIManager.setLookAndFeel(
        "com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
      } catch (Exception e) {
          e.printStackTrace();
      }
		
		JPanel contentPane = (JPanel) this.getContentPane();
		contentPane.setLayout(new BorderLayout());
    this.setSize(new Dimension(640, 480));
    this.setTitle("Terrain Smoother");
    
    //Creates a terrain canvas
		terrainCanvas = new TerrainCanvas(this.getSize().width, this.getSize().height);
		
		//Create a terrainLoader
		terrainLoader = new TerrainLoader(file_path);
		
		//Creates objects of all the smoothing filters.
		bandFilter = new BandFilter(terrainLoader.getTerrain(), 0.2f);
		lowpassFilter = new LowpassFilter(terrainLoader.getTerrain(), 0.9f);
		highpassFilter = new HighpassFilter(terrainLoader.getTerrain(), 0.9f);
		meanFilter = new MeanFilter(terrainLoader.getTerrain());
		gaussianFilter = new GaussianFilter(terrainLoader.getTerrain());
		kuwaharaFilter = new KuwaharaFilter(terrainLoader.getTerrain());
		
		heightField = new HeightField(terrainLoader.getTerrain());
		
		terrainCanvas.setHeightField(heightField);
		
		setupInputGUI();
		setupStatusBar();
		
		contentPane.add("North", input_panel);
		input_panel.setVisible(false);
		contentPane.add("Center", terrainCanvas);
		contentPane.add("South", statusBar);
    
    winManager = new WindowManager();
    this.addWindowListener(winManager);
    
    mouseManager = new MouseManager(terrainCanvas);
    terrainCanvas.addMouseListener(mouseManager);
		terrainCanvas.addMouseMotionListener(mouseManager);
    
    keyManager = new KeyManager(terrainCanvas);
    terrainCanvas.addKeyListener(keyManager);
    
    setupMenuBar();
    setMenuBar(viewMenuBar);
    
    this.pack();
    this.show();
    
    terrainCanvas.start();
	}
	
	/**
	 * smooth
	 * @param int iterations - determine how many times the filter should be applied to the heightfield.
	 * Smooth the heightfield.
	 */
	private void smooth(int iterations){ 
		filter.setMatrix(heightField.getHeightField());
		message_label.setText(filter.getFilterName());
		
		for(int i = 0; i < iterations; i++) 
			filter.smooth(); 
		
		heightField.setHeightField(filter.getMatrix());
		terrainCanvas.repaint(); 
	}
	
	/**
	 * setupMenuBar
	 * Creates a menubar
	 */
	private void setupMenuBar(){
		viewMenuBar = new MenuBar();
    
    Menu file_menu = new Menu("File");
    	open_item = new MenuItem("Open");
    	open_item.addActionListener(this);
    	exit_item = new MenuItem("Exit");
    	exit_item.addActionListener(this);
    	
    	file_menu.add(open_item);
    	file_menu.add(exit_item);
    	
    Menu filter_menu = new Menu("Filter");
    	bandfilter_item = new MenuItem("BandFilter");
    	lowpassfilter_item = new MenuItem("LowpassFilter");
    	highfilter_item = new MenuItem("HighpassFilter");
    	meanfilter_item = new MenuItem("MeanFilter");
    	gaussianfilter_item = new MenuItem("GaussianFilter");
    	kuwaharafilter_item = new MenuItem("KuwaharaFilter");
    	remove_filtering_item = new MenuItem("Remove filtering");
    	
    	bandfilter_item.addActionListener(this);
    	lowpassfilter_item.addActionListener(this);
    	highfilter_item.addActionListener(this);
    	meanfilter_item.addActionListener(this);
    	gaussianfilter_item.addActionListener(this);
    	kuwaharafilter_item.addActionListener(this);
    	remove_filtering_item.addActionListener(this);
    	
    	filter_menu.add(bandfilter_item);
    	filter_menu.add(lowpassfilter_item);
    	filter_menu.add(highfilter_item);
    	filter_menu.add(meanfilter_item);
    	filter_menu.add(gaussianfilter_item);
    	filter_menu.add(kuwaharafilter_item);
    	filter_menu.addSeparator();
    	filter_menu.add(remove_filtering_item);
    
    viewMenuBar.add(file_menu);
    viewMenuBar.add(filter_menu);
	}
	
	/**
	 * setupInputGUI
	 * setup the filter panel. 
	 */
	private void setupInputGUI(){
		input_panel = new JPanel(new FlowLayout(FlowLayout.LEADING, 0, 0));
		
		input_label = new JLabel("");
		input_field = new JTextField(2);
		
		iteration_label = new JLabel(" Iterations: ");
		iteration_field = new JTextField(2);
		
		smooth_button = new JButton("Smooth");
		smooth_button.addActionListener(this);
		
		input_panel.add(input_label);
		input_panel.add(input_field);
		input_panel.add(iteration_label);
		input_panel.add(iteration_field);
		input_panel.add(smooth_button);
		
	}
	
	/**
	 * setupStatusBar
	 * setup the status bar in the bottom of the window.
	 */
	private void setupStatusBar(){
		statusBar = new JPanel(new FlowLayout(FlowLayout.LEADING, 0, 0));
		message_label = new JLabel("No filter selected");
		statusBar.add(message_label);
	}
	
	/**
	 * actionPerformed
	 * @param ActionEvent e
	 * This method listen to the actionevents in the window and 
	 * produce different procedures when a action is recognised.
	 */
	public void actionPerformed(ActionEvent e){
		if(e.getSource() == exit_item){
			System.exit(0);
		}
		else if(e.getSource() == open_item){
			FileDialog file_chooser = new FileDialog(this, "Select a terrain file", FileDialog.LOAD);
			
			file_chooser.setDirectory("d:\\halden\\vaar04\\grafdat\\terrain\\");
			
			file_chooser.show();
			
			file_path = file_chooser.getDirectory() + file_chooser.getFile();
			
			terrainLoader.open(file_path);
			heightField.setHeightField(terrainLoader.getTerrain());
			
			terrainCanvas.setHeightField(heightField);
 			terrainCanvas.repaint();
 
		}
		else if(e.getSource() == bandfilter_item){
			filter = bandFilter;
			input_label.setVisible(true);
			input_field.setVisible(true);
			input_panel.setVisible(true);
			input_label.setText("Set K value: ");
		}
		else if(e.getSource() == lowpassfilter_item){
			filter = lowpassFilter;
			input_label.setVisible(true);
			input_field.setVisible(true);
			input_label.setText("Set Frequency value: ");
			input_panel.setVisible(true);
		}
		else if(e.getSource() == highfilter_item){
			filter = highpassFilter;
			input_label.setVisible(true);
			input_field.setVisible(true);
			input_label.setText("Set Frequency value: ");
			input_panel.setVisible(true);
		}
		else if(e.getSource() == meanfilter_item){
			filter = meanFilter;
			input_label.setVisible(false);
			input_field.setVisible(false);
			input_panel.setVisible(true);
		}
		else if(e.getSource() == gaussianfilter_item){
			filter = gaussianFilter;
			input_label.setVisible(false);
			input_field.setVisible(false);
			input_label.setText("Set Sigma value: ");
			input_panel.setVisible(true);
		}
		else if(e.getSource() == kuwaharafilter_item){
			filter = kuwaharaFilter;
			input_label.setVisible(false);
			input_field.setVisible(false);
			input_panel.setVisible(true);
		}
		else if(e.getSource() == remove_filtering_item){
			input_panel.setVisible(false);
			message_label.setText("No filter selected.");
			terrainLoader.open(file_path);
			heightField.setHeightField(terrainLoader.getTerrain());

			terrainCanvas.setHeightField(heightField);
 			terrainCanvas.repaint();
		}
		
		if(e.getSource() == smooth_button){
			if(iteration_field.getText().equals(""))
				iteration_field.setText("1");
			if(input_field.getText().equals(""))
				input_field.setText("0.1");
			
			filter.setParameter(Float.parseFloat(input_field.getText()));
			smooth(Integer.parseInt(iteration_field.getText()));
		}
	}
}