package terrain.filter;

/******************************
 * Kuwahara filter
 * date: 2004-05-26
 * author: Kristian Lunde
 * This filter divide the heightField into 4 regions, the heightField is a square where x=y=4L+1 
 * each region are defined: [(x+1)/2]*[(y+1)/2].
 *****************************/

public class KuwaharaFilter extends Filter{

	private int kernel_size;
	/* Define how many windows/kernels that should be used.*/
	private int kernels = 4;
	private boolean correct_size;
	
	private MeanFilter meanFilter;

	/*Constructor*/
	public KuwaharaFilter(){ meanFilter = new MeanFilter(); }

	/**
	 * Constructor
	 * @param float[][][] matrix - a matrix of height values.
	 */
	public KuwaharaFilter(float[][][] matrix){
		setMatrix(matrix);
		checkMatrixSize();
		kernel_size = (getMatrixSize() + 1) / 2;
		meanFilter = new MeanFilter();
	}
	
	/**
	 * smooth
	 * Smooth the heightfield.
	 */
	public void smooth(){
		kernel_size = (getMatrixSize() + 1) / 2;
		
		if(correct_size){
			int tmpX = 0;
			int tmpY = 0;
		
			for(int h = 0 ; h < kernels / 2; h++){
				for(int v = 0; v < kernels / 2; v++){
				
					float kernel[][][]  = getKernel(tmpX, tmpY);
				
					//find mean to all the points in the kernel.
					meanFilter.setMatrix(kernel);
					meanFilter.smooth();
					kernel = meanFilter.getMatrix();
				
					float var[][][] = kernelVariance(kernel, kernelMean(kernel));
				
					kernel = replace(kernel, var);
				
					insertNewValues(kernel, tmpX, tmpY);
				
					tmpX = kernel_size - 1;	
				
				}
				tmpX = 0;
				tmpY = kernel_size - 1;	
			}
		}
	}
	
	//Retrieve a kernel of size kernel_size from a matrix.
	/**
	 * getKernel
	 * @param int tx - define the x start position in the heightfield, where the kernel should start.
	 * @param int ty - define the y start position in the heightfield, where the kernel should start.
	 * @return float[][][] - return the kernel of size kernel_size * kernel_size;
	 */
	private float[][][] getKernel(int tx, int ty){
		float[][][] kernel = new float[kernel_size][kernel_size][3];
		int i = 0;	//Set the kernel x index to 0;
		for(int x = tx; x < tx + kernel_size; x++){
			int j = 0; //Set the kernel y index to 0;
			for(int y = ty; y < ty + kernel_size; y++){
				kernel[i][j][0] = matrix[x][y][0];
				kernel[i][j][1] = matrix[x][y][1];
				kernel[i][j][2] = matrix[x][y][2];
				j++;
			}
			i++;
		}
		return kernel;
	}
	
	/**
	 * kernelMean
	 * @param float[][][] kernel - is the kernel to calculate the mean of.
	 * @return float - return the mean value of the kernel.
	 */
	private float kernelMean(float[][][] kernel){
		float mean = 0.0f;
		for(int x = 0; x < kernel_size; x++){
			for(int y = 0; y < kernel_size; y++){
				mean += kernel[x][y][2];
			}
		}
		return mean / (kernel_size * kernel_size);
	}
	
	//MERK: mange verdier blir godt under null! dette er vel strengt talt ikke lov.
	/**
	 * kernelVariance
	 * @param float[][][] kernel - is the kernel to calculate the variance of.
	 * @param float kernel_mean - is the mean of the kernel.
	 * @return float[][][] - return a array of variance to each heightpoint in the kernel.
	 */
	private float[][][] kernelVariance(float[][][] kernel, float kernel_mean){
		float kernel_var[][][] = new float[kernel_size][kernel_size][1];
		
		for(int x = 0; x < kernel_size; x++){
			for(int y = 0; y < kernel_size; y++){
				kernel_var[x][y][0] = kernel_mean - (kernel[x][y][2] * kernel[x][y][2]);
			}
		}
		return kernel_var;
	}
	
	/**
	 * replace
	 * @param float[][][] kernel - is the kernel that contain the new values for a area in the heightfield.
	 * @param float[][][] lowest_variance - contain all the variances to the kernel.
	 * @return float[][][] - return a kernel where all heightvalues has the same value as the one with lowest variance.
	 */
	private float[][][] replace(float[][][] kernel, float[][][] lowest_variance){
		float var = 5.0f;
		int tx = 0;
		int ty = 0;
		
		//Find the lowest variance of the variances in the kernel.
		for(int x = 0; x < kernel_size; x++){
			for(int y = 0; y < kernel_size; y++){
				if(compareTo(var, lowest_variance[x][y][0])){
					var = lowest_variance[x][y][0];
					tx = x;
					ty = y;
				}
			}	
		}
		
		//Restrict the lowest variance to be between -3.0 and 3.0.
		if(lowest_variance[tx][ty][0] < -3.0f)
			lowest_variance[tx][ty][0] = -3.0f;
		else if(lowest_variance[tx][ty][0] > 3.0f)
			lowest_variance[tx][ty][0] = 3.0f;
		
		//Replace the kernel heightvalues with the one with lowest_variance.
		for(int x = 0; x < kernel_size; x++){
			for(int y = 0; y < kernel_size; y++){
				kernel[x][y][2] = lowest_variance[tx][ty][0];
			}
		}
		return kernel;
	}
	
	/**
	 * makeMatrix
	 * @param float[] kernels - contain all the kernels that should make the whole matrix.
	 * Anticipate that the kernels are horizontal sorted before vertical sorted
	 */
	//private void makeMatrix(float[] kernels){
	
	/**
	 * insertNewValues
	 * @param float[][][] kernel - is the kernel with the new calculated heightvalues.
	 * @param int kx - is the start x position of where the kernel should be placed in the heightfield.
	 * @param int ky - is the start y position of where the kernel should be placed in the heightfield.
	 * This method anticipate that it is 4 kernels/windows.
	 */
	private void insertNewValues(float[][][] kernel, int kx, int ky){	
		int i = 0;	//Set the kernel x index to 0;
		for(int x = kx; x < kx + kernel_size; x++){
			int j = 0; //Set the kernel y index to 0;
			for(int y = ky; y < ky + kernel_size; y++){
				matrix[x][y][2] = kernel[i][j][2];
				j++;
			}
			i++;
		}
	}
	
	/**
	 * compareTo
	 * @param float a - is this number larger than b? if so return true.
	 * @param float b - return false if b is larger than a.
	 * @return boolean - true if a is larger than b.
	 * This method are used by the replace method to determine the largest of to variances.
	 */
	private boolean compareTo(float a, float b){
		if(isNegative(a))
			a = a * (-1.0f);
		if(isNegative(b))
			b = b *(-1.0f);
		
		return a > b;
	}
	
	/**
	 * isNegative
	 * @param float a - is a number, might be negative
	 * @return boolean - true if a is less than zero.
	 * This method is used by the compareTo method.
	 */
	private boolean isNegative(float a){ return (a < 0.0f); }
	
	/**
	 * checkMatrixSize
	 * Check to see if the matrix has the right size to use a kuwaharafilter.
	 */
	private void checkMatrixSize(){
		correct_size = true;
		filter_name = "Kuwahara filter";
		
		if(getMatrixSize() < 5){
			filter_name = "Warning! to run a Kuwahara filter the TerrainField must have atleast a 5x5 size.";
			correct_size = false;
		}
		else if(getMatrixSize() % 2 != 1){
			filter_name = "Warning! The terrainField size can not be dividable on 2.";
			correct_size = false;
		}
	}
	
	/**
	 * setMatrix
	 * @param float[][][] m - is the heightfield matrix.
	 * Reimplements the setMatrix method from the superclass
	 */
	public void setMatrix(float[][][] m){
		matrix = m;
		matrix_size = m.length;
		checkMatrixSize();
	}
}