package terrain.filter;

/****************************
 * Mean filter
 * date: 2004-05-20
 * author: Kristian Lunde
 * description:
 * A mean filter are based on the neighbour points and compute a average point value
 * based on the neighbours. Since the mean filter uses the neighbours it is important to 
 * define the size of the neighbour matrix. 
 * This implementation use a  3x3 neighbour matrix.
 * Edgepoints will only be based on 3 and six neigbours in a 3x3 neighboursize.
 ***************************/
 
public class MeanFilter extends Filter {

	private static final int UPPER_LEFT = 0;
	private static final int UPPER_RIGHT = 1;
	private static final int LOWER_LEFT = 2;
	private static final int LOWER_RIGHT = 3;

	private static final int KERNEL_SIZE = 3;

	/* Constructor */
	public MeanFilter(){}

	/**
	 * Constructor
	 * @param float[][][] matrix - set the heightfield matrix to be smoothed.
	 */
	public MeanFilter(float[][][] matrix){ 
		super(matrix);
		filter_name = "Mean filter";
	}
	
	/**
	 * smooth
	 * This method do the actuall smoothing of the matrix.
	 */
	public void smooth(){
		for(int x = 1; x < matrix.length - 1; x++){
			for(int y = 1; y < matrix[x].length -1; y++){
				matrix[x][y][2] = center(x, y);
			}
			//Calculate average edge values.
			matrix[0][x][2] = verticalEdge(0, x, false);
			matrix[getMatrixSize()-1][x][2] = verticalEdge(getMatrixSize() -1, x, true);
			matrix[x][0][2] = horizontalEdge(x, 0, false); 
			matrix[x][getMatrixSize() - 1][2] = horizontalEdge(x, getMatrixSize() - 1, true);
		}
		//Calculate corner values
		matrix[0][0][2] = corner(UPPER_LEFT);
		matrix[getMatrixSize() - 1][0][2] = corner(UPPER_RIGHT);
		matrix[0][getMatrixSize() - 1][2] = corner(LOWER_LEFT);
		matrix[getMatrixSize() - 1][getMatrixSize() - 1][2] = corner(LOWER_RIGHT); //lowerRightEdge();
	}
	
	/**
	 * center
	 * @param int x - is the x coordinate of the point to be averaged.
	 * @param int y - is the y coordinate of the point to be averaged.
	 * @return float - the average value of the heightpoint.
	 */
	private float center(int x, int y){
		float average = 0.0f;
		x = x - 1;
		y = y - 1;
		
		for(int tx = x; tx <  x + KERNEL_SIZE; tx++){
			for(int ty = y; ty < y + KERNEL_SIZE; ty++)
				average += matrix[tx][ty][2];
		}
		return average / (KERNEL_SIZE * KERNEL_SIZE);
	}
	
	//Calculate the average of a point on one of the horizontal edges of the matrix.
		/**
	 * horizontalEdge
	 * @param int x - is the x coordinate of the point to be averaged.
	 * @param int y - is the y coordinate of the point to be averaged.
	 * @param boolean bottom - if true it is a edgepoint on the bottom of the matrix.
	 * @return float - the average value of the heightpoint.
	 */
	private float horizontalEdge(int x, int y, boolean bottom){
		float average = 0.0f;
		x = x - 1;
		if(bottom) y = y - 1;
		
		for(int tx = x; tx < x + KERNEL_SIZE; tx++){
			for(int ty = y; ty <  y + KERNEL_SIZE - 1; ty++)
				average += matrix[tx][ty][2];
		}
		return average / (KERNEL_SIZE *(KERNEL_SIZE -1));
	}
	
	/**
	 * verticalEgde
	 * @param int x - the heightvalue's x position.
	 * @param int y - the heightvalue's y position.
	 * @param boolean right - position the vertical edge to either right if it is true else left.
	 * @return float - the average value for the heightposition.
	 */
	private float verticalEdge(int x, int y, boolean right){
		float average = 0.0f;
		y = y - 1; 
		if(right) x = x - 1;
		
		for(int tx = x; tx <  x + KERNEL_SIZE - 1; tx++){
			for(int ty = y; ty < y + KERNEL_SIZE; ty++) 
				average += matrix[tx][ty][2];
		}
		return average / (KERNEL_SIZE * (KERNEL_SIZE -1 ));
	}
	
	/**
	 * corner
	 * @param int corner - define what corner should be calculated
	 * @return float - the average of the corner.
	 * This method calculate the mean value of a corner in the heightfield.
	 */
	private float corner(int corner){
		float average = 0.0f;
		int x = 0;
		int y = 0;
		int x_size = 0;
		int y_size = 0;
		
		if(corner == UPPER_LEFT){
			x_size = KERNEL_SIZE - 1;
			y_size = KERNEL_SIZE - 1;
			x = 0;
			y = 0;
		}
		else if(corner == UPPER_RIGHT){
			x_size = getMatrixSize();
			y_size = KERNEL_SIZE - 1;
			x = matrix.length - (KERNEL_SIZE - 1);
			y = 0;
		}
		else if(corner == LOWER_LEFT){
			x_size = KERNEL_SIZE - 1;
			y_size = getMatrixSize();
			x = 0;
			y = getMatrixSize() - (KERNEL_SIZE - 1);
		}
		else if(corner == LOWER_RIGHT){
			x_size = getMatrixSize();
			y_size = getMatrixSize();
			x = getMatrixSize() - (KERNEL_SIZE - 1);
			y = getMatrixSize() - (KERNEL_SIZE - 1);
		}
		
		for(int tx = x; tx < x_size; tx++){
			for(int ty = y; ty < y_size; ty++)
				average += matrix[tx][ty][2];			
		}
		return average / ((KERNEL_SIZE - 1) * (KERNEL_SIZE - 1));
	}
}